@@ -12,7 +12,6 @@
 #include "ksycoca.h"
 
 #include "ksycocadict_p.h"
-#include "ksycocaresourcelist_p.h"
 #include "sycocadebug.h"
 #include <KDesktopFile>
 
@@ -21,17 +20,16 @@
 #include <QMimeDatabase>
 
 #include <QStandardPaths>
-#include <assert.h>
 #include <kmimetypefactory_p.h>
+#include <kservice_p.h>
 
-KBuildServiceFactory::KBuildServiceFactory(KBuildMimeTypeFactory *mimeTypeFactory, KBuildServiceGroupFactory *serviceGroupFactory)
+KBuildServiceFactory::KBuildServiceFactory(KBuildMimeTypeFactory *mimeTypeFactory)
     : KServiceFactory(mimeTypeFactory->sycoca())
     , m_nameMemoryHash()
     , m_relNameMemoryHash()
     , m_menuIdMemoryHash()
     , m_dupeDict()
     , m_mimeTypeFactory(mimeTypeFactory)
-    , m_serviceGroupFactory(serviceGroupFactory)
 {
     m_nameDict = new KSycocaDict();
     m_relNameDict = new KSycocaDict();
@@ -217,29 +215,27 @@ void KBuildServiceFactory::populateServiceTypes()
         KService::Ptr service(static_cast<KService *>(servIt.value().data()));
         const bool hidden = !service->showInCurrentDesktop();
 
-        QList<KService::ServiceTypeAndPreference> serviceTypeList = service->_k_accessServiceTypes();
+        const auto mimeTypes = service->d_func()->m_mimeTypes;
 
         // Add this service to all its MIME types
         // Don't cache count(), it can change during iteration! (we can't use an iterator-based loop
         // here the container could get reallocated which would invalidate iterators)
-        for (int i = 0; i < serviceTypeList.count(); ++i) {
-            const KService::ServiceTypeAndPreference &typeAndPref = serviceTypeList.at(i);
-            const QString stName = typeAndPref.serviceType;
+        for (int i = 0; i < mimeTypes.count(); ++i) {
+            const QString &mimeName = mimeTypes.at(i);
 
             if (hidden) {
                 continue;
             }
-            const int preference = typeAndPref.preference;
 
-            KServiceOffer offer(service, preference, 0);
-            QMimeType mime = db.mimeTypeForName(stName);
+            KServiceOffer offer(service, 1 /* preference; always 1 here, may be higher based on KMimeAssociations */, 0);
+            QMimeType mime = db.mimeTypeForName(mimeName);
             if (!mime.isValid()) {
-                if (stName.startsWith(QLatin1String("x-scheme-handler/"))) {
+                if (mimeName.startsWith(QLatin1String("x-scheme-handler/"))) {
                     // Create those on demand
-                    m_mimeTypeFactory->createFakeMimeType(stName);
-                    m_offerHash.addServiceOffer(stName, offer);
+                    m_mimeTypeFactory->createFakeMimeType(mimeName);
+                    m_offerHash.addServiceOffer(mimeName, offer);
                 } else {
-                    qCDebug(SYCOCA) << service->entryPath() << "specifies undefined MIME type/servicetype" << stName;
+                    qCDebug(SYCOCA) << service->entryPath() << "specifies undefined MIME type/servicetype" << mimeName;
                     // technically we could call addServiceOffer here, 'mime' isn't used. But it
                     // would be useless, since we have no MIME type entry where to write the offers offset.
                     continue;
@@ -250,7 +246,7 @@ void KBuildServiceFactory::populateServiceTypes()
 
                 for (const QString &otherType : lst) {
                     // Skip derived types if the base class is listed (#321706)
-                    if (stName != otherType && mime.inherits(otherType)) {
+                    if (mimeName != otherType && mime.inherits(otherType)) {
                         // But don't skip aliases (they got resolved into mime.name() already, but don't let two aliases cancel out)
                         if (db.mimeTypeForName(otherType).name() != mime.name()) {
                             // qCDebug(SYCOCA) << "Skipping" << mime.name() << "because of" << otherType << "(canonical" << db.mimeTypeForName(otherType) <<
